@@ -1,8 +1,7 @@
 C Output from Public domain Ratfor, version 1.01
       subroutine llrmnewton (cd, nxis, q, nxi, rs, nobs, cntsum, cnt, qd
-     *rs, nqd, nx, xxwt, idx, prec, maxiter, mchpr, wk, info)
-      integer nxis, nxi, nobs, cntsum, cnt(*), nqd, nx, idx(*), maxiter,
-     * info
+     *rs, nqd, nx, xxwt, prec, maxiter, mchpr, wk, info)
+      integer nxis, nxi, nobs, cntsum, cnt(*), nqd, nx, maxiter, info
       double precision cd(*), q(nxi,*), rs(nxis,*), qdrs(nqd,nxis,*), xx
      *wt(*), prec, mchpr, wk(*)
       integer iwt, iwtsum, imrs, ifit, imu, imuwk, iv, ivwk, ijpvt, icdn
@@ -22,24 +21,23 @@ C Output from Public domain Ratfor, version 1.01
       ifitnew = iwtnewsum + nx
       iwk = ifitnew + nobs
       call llrmnewton1 (cd, nxis, q, nxi, rs, nobs, cntsum, cnt, qdrs, n
-     *qd, nx, xxwt, idx, prec, maxiter, mchpr, wk(iwt), wk(iwtsum), wk(i
-     *mrs), wk(ifit), wk(imu), wk(imuwk), wk(iv), wk(ivwk), wk(ijpvt), w
-     *k(icdnew), wk(iwtnew), wk(iwtnewsum), wk(ifitnew), wk(iwk), info)
+     *qd, nx, xxwt, prec, maxiter, mchpr, wk(iwt), wk(iwtsum), wk(imrs),
+     * wk(ifit), wk(imu), wk(imuwk), wk(iv), wk(ivwk), wk(ijpvt), wk(icd
+     *new), wk(iwtnew), wk(iwtnewsum), wk(ifitnew), wk(iwk), info)
       return
       end
       subroutine llrmnewton1 (cd, nxis, q, nxi, rs, nobs, cntsum, cnt, q
-     *drs, nqd, nx, xxwt, idx, prec, maxiter, mchpr, wt, wtsum, mrs, fit
-     *, mu, muwk, v, vwk, jpvt, cdnew, wtnew, wtnewsum, fitnew, wk, info
-     *)
-      integer nxis, nxi, nobs, cntsum, cnt(*), nqd, nx, idx(*), maxiter,
-     * jpvt(*), info
+     *drs, nqd, nx, xxwt, prec, maxiter, mchpr, wt, wtsum, mrs, fit, mu,
+     * muwk, v, vwk, jpvt, cdnew, wtnew, wtnewsum, fitnew, wk, info)
+      integer nxis, nxi, nobs, cntsum, cnt(*), nqd, nx, maxiter, jpvt(*)
+     *, info
       double precision cd(*), q(nxi,*), rs(nxis,*), qdrs(nqd,nxis,*), xx
      *wt(*), prec, mchpr, wt(nqd,*), wtsum(*), mrs(*), fit(*), mu(*), mu
      *wk(*), v(nxis,*), vwk(nxis,*), cdnew(*), wtnew(nqd,*), wtnewsum(*)
      *, fitnew(*), wk(*)
       integer i, j, k, kk, iter, flag, rkv, idamax, infowk
-      double precision tmp, ddot, fitmean, lkhd, mumax, lkhdnew, disc, d
-     *isc0, trc
+      double precision norm, tmp, ddot, fitmean, lkhd, mumax, lkhdnew, d
+     *isc, disc0, trc
       info = 0
       i=1
 23000 if(.not.(i.le.nxis))goto 23002
@@ -69,6 +67,7 @@ C Output from Public domain Ratfor, version 1.01
       else
       trc = 1.d0 / dfloat (cntsum)
       endif
+      norm = 0.d0
       kk=1
 23013 if(.not.(kk.le.nx))goto 23015
       wtsum(kk) = 0.d0
@@ -79,13 +78,14 @@ C Output from Public domain Ratfor, version 1.01
 23017 i=i+1
       goto 23016
 23018 continue
+      norm = norm + xxwt(kk) * dlog (wtsum(kk))
 23014 kk=kk+1
       goto 23013
 23015 continue
       fitmean = 0.d0
       i=1
 23019 if(.not.(i.le.nobs))goto 23021
-      tmp = ddot (nxis, rs(1,i), 1, cd, 1) - dlog (wtsum(idx(i)))
+      tmp = ddot (nxis, rs(1,i), 1, cd, 1)
       fit(i) = dexp (tmp)
       if(cntsum.ne.0)then
       tmp = tmp * dfloat (cnt(i))
@@ -95,7 +95,8 @@ C Output from Public domain Ratfor, version 1.01
       goto 23019
 23021 continue
       call dsymv ('u', nxi, 1.d0, q, nxi, cd, 1, 0.d0, wk, 1)
-      lkhd = lkhd + ddot (nxi, cd, 1, wk, 1) / 2.d0 - fitmean * trc
+      lkhd = lkhd + ddot (nxi, cd, 1, wk, 1) / 2.d0 - fitmean * trc + no
+     *rm
       iter = 0
       flag = 0
 23024 continue
@@ -174,6 +175,7 @@ C Output from Public domain Ratfor, version 1.01
       call dtrsl (v, nxis, nxis, cdnew, 01, infowk)
       call dprmut (cdnew, nxis, jpvt, 1)
       call daxpy (nxis, 1.d0, cd, 1, cdnew, 1)
+      norm = 0.d0
       kk=1
 23059 if(.not.(kk.le.nx))goto 23061
       wtnewsum(kk) = 0.d0
@@ -184,6 +186,7 @@ C Output from Public domain Ratfor, version 1.01
 23063 i=i+1
       goto 23062
 23064 continue
+      norm = norm + xxwt(kk) * dlog (wtnewsum(kk))
 23060 kk=kk+1
       goto 23059
 23061 continue
@@ -191,7 +194,7 @@ C Output from Public domain Ratfor, version 1.01
       fitmean = 0.d0
       i=1
 23067 if(.not.(i.le.nobs))goto 23069
-      tmp = ddot (nxis, rs(1,i), 1, cdnew, 1) - dlog (wtnewsum(idx(i)))
+      tmp = ddot (nxis, rs(1,i), 1, cdnew, 1)
       if(tmp.gt.3.d2)then
       flag = flag + 1
       goto 23069
@@ -205,7 +208,8 @@ C Output from Public domain Ratfor, version 1.01
       goto 23067
 23069 continue
       call dsymv ('u', nxi, 1.d0, q, nxi, cdnew, 1, 0.d0, wk, 1)
-      lkhdnew = ddot (nxi, cdnew, 1, wk, 1) / 2.d0 - fitmean * trc
+      lkhdnew = ddot (nxi, cdnew, 1, wk, 1) / 2.d0 - fitmean * trc + nor
+     *m
       endif
       if(flag.eq.1)then
       call dset (nxis, 0.d0, cd, 1)
@@ -323,6 +327,12 @@ C Output from Public domain Ratfor, version 1.01
 23115 continue
       lkhd = lkhd / dfloat (cntsum)
       endif
+      kk=1
+23116 if(.not.(kk.le.nx))goto 23118
+      lkhd = lkhd - xxwt(kk) * dlog (wtsum(kk))
+23117 kk=kk+1
+      goto 23116
+23118 continue
       wtsum(1) = lkhd
       wtsum(2) = trc
       return
@@ -335,79 +345,79 @@ C Output from Public domain Ratfor, version 1.01
       integer i, j, k, kk, rkv
       double precision ddot
       kk=1
-23116 if(.not.(kk.le.nx))goto 23118
+23119 if(.not.(kk.le.nx))goto 23121
       wtsum(kk) = 0.d0
       i=1
-23119 if(.not.(i.le.nqd))goto 23121
+23122 if(.not.(i.le.nqd))goto 23124
       wt(i,kk) = dexp (ddot (nxis, qdrs(i,1,kk), nqd, cd, 1))
       wtsum(kk) = wtsum(kk) + wt(i,kk)
-23120 i=i+1
+23123 i=i+1
+      goto 23122
+23124 continue
+23120 kk=kk+1
       goto 23119
 23121 continue
-23117 kk=kk+1
-      goto 23116
-23118 continue
       call dset (nxis*nxis, 0.d0, v, 1)
       kk=1
-23122 if(.not.(kk.le.nx))goto 23124
-      i=1
-23125 if(.not.(i.le.nxis))goto 23127
-      mu(i) = ddot (nqd, wt(1,kk), 1, qdrs(1,i,kk), 1) / wtsum(kk)
-23126 i=i+1
-      goto 23125
-23127 continue
+23125 if(.not.(kk.le.nx))goto 23127
       i=1
 23128 if(.not.(i.le.nxis))goto 23130
-      j=i
-23131 if(.not.(j.le.nxis))goto 23133
-      vwk(i,j) = 0.d0
-      k=1
-23134 if(.not.(k.le.nqd))goto 23136
-      vwk(i,j) = vwk(i,j) + wt(k,kk) * qdrs(k,i,kk) * qdrs(k,j,kk)
-23135 k=k+1
-      goto 23134
-23136 continue
-      vwk(i,j) = vwk(i,j) / wtsum(kk) - mu(i) * mu(j)
-23132 j=j+1
-      goto 23131
-23133 continue
+      mu(i) = ddot (nqd, wt(1,kk), 1, qdrs(1,i,kk), 1) / wtsum(kk)
 23129 i=i+1
       goto 23128
 23130 continue
-      call daxpy (nxis*nxis, xxwt(kk), vwk, 1, v, 1)
-23123 kk=kk+1
-      goto 23122
-23124 continue
       i=1
-23137 if(.not.(i.le.nxi))goto 23139
+23131 if(.not.(i.le.nxis))goto 23133
       j=i
-23140 if(.not.(j.le.nxi))goto 23142
-      v(i,j) = v(i,j) + q(i,j)
-23141 j=j+1
-      goto 23140
-23142 continue
-23138 i=i+1
+23134 if(.not.(j.le.nxis))goto 23136
+      vwk(i,j) = 0.d0
+      k=1
+23137 if(.not.(k.le.nqd))goto 23139
+      vwk(i,j) = vwk(i,j) + wt(k,kk) * qdrs(k,i,kk) * qdrs(k,j,kk)
+23138 k=k+1
       goto 23137
 23139 continue
+      vwk(i,j) = vwk(i,j) / wtsum(kk) - mu(i) * mu(j)
+23135 j=j+1
+      goto 23134
+23136 continue
+23132 i=i+1
+      goto 23131
+23133 continue
+      call daxpy (nxis*nxis, xxwt(kk), vwk, 1, v, 1)
+23126 kk=kk+1
+      goto 23125
+23127 continue
       i=1
-23143 if(.not.(i.le.nxis))goto 23145
-      jpvt(i) = 0
-23144 i=i+1
+23140 if(.not.(i.le.nxi))goto 23142
+      j=i
+23143 if(.not.(j.le.nxi))goto 23145
+      v(i,j) = v(i,j) + q(i,j)
+23144 j=j+1
       goto 23143
 23145 continue
-      call dchdc (v, nxis, nxis, vwk, jpvt, 1, rkv)
-23146 if(v(rkv,rkv).lt.v(1,1)*dsqrt(mchpr))then
-      rkv = rkv - 1
+23141 i=i+1
+      goto 23140
+23142 continue
+      i=1
+23146 if(.not.(i.le.nxis))goto 23148
+      jpvt(i) = 0
+23147 i=i+1
       goto 23146
+23148 continue
+      call dchdc (v, nxis, nxis, vwk, jpvt, 1, rkv)
+23149 if(v(rkv,rkv).lt.v(1,1)*dsqrt(mchpr))then
+      rkv = rkv - 1
+      goto 23149
       endif
-23147 continue
+23150 continue
       i=rkv+1
-23148 if(.not.(i.le.nxis))goto 23150
+23151 if(.not.(i.le.nxis))goto 23153
       v(i,i) = v(1,1)
       call dset (i-rkv-1, 0.d0, v(rkv+1,i), 1)
-23149 i=i+1
-      goto 23148
-23150 continue
+23152 i=i+1
+      goto 23151
+23153 continue
       return
       end
       subroutine llrmrkl (cd, nxis, qdrs, nqd, nx, xxwt, wt0, offset, mc
@@ -420,79 +430,79 @@ C Output from Public domain Ratfor, version 1.01
       integer i, j, k, kk, iter, flag, idamax, infowk
       double precision ddot, dasum, rkl, tmp, mumax, rklnew, disc, disc0
       kk=1
-23151 if(.not.(kk.le.nx))goto 23153
+23154 if(.not.(kk.le.nx))goto 23156
       i=1
-23154 if(.not.(i.le.nqd))goto 23156
+23157 if(.not.(i.le.nqd))goto 23159
       wt(i,kk) = dexp (ddot (nxis, qdrs(i,1,kk), nqd, cd, 1) + offset(i,
      *kk))
-23155 i=i+1
-      goto 23154
-23156 continue
-      call dscal (nqd, 1.d0/dasum(nqd,wt(1,kk),1), wt(1,kk), 1)
-23152 kk=kk+1
-      goto 23151
-23153 continue
-      rkl = 0.d0
-      kk=1
-23157 if(.not.(kk.le.nx))goto 23159
-      tmp = 0.d0
-      i=1
-23160 if(.not.(i.le.nqd))goto 23162
-      tmp = tmp + dlog(wt0(i,kk)/wt(i,kk)) * wt0(i,kk)
-23161 i=i+1
-      goto 23160
-23162 continue
-      rkl = rkl + xxwt(kk) * tmp
-23158 kk=kk+1
+23158 i=i+1
       goto 23157
 23159 continue
+      call dscal (nqd, 1.d0/dasum(nqd,wt(1,kk),1), wt(1,kk), 1)
+23155 kk=kk+1
+      goto 23154
+23156 continue
+      rkl = 0.d0
+      kk=1
+23160 if(.not.(kk.le.nx))goto 23162
+      tmp = 0.d0
+      i=1
+23163 if(.not.(i.le.nqd))goto 23165
+      tmp = tmp + dlog(wt0(i,kk)/wt(i,kk)) * wt0(i,kk)
+23164 i=i+1
+      goto 23163
+23165 continue
+      rkl = rkl + xxwt(kk) * tmp
+23161 kk=kk+1
+      goto 23160
+23162 continue
       iter = 0
       flag = 0
-23163 continue
+23166 continue
       iter = iter + 1
       call dset (nxis, 0.d0, mu, 1)
       call dset (nxis*nxis, 0.d0, v, 1)
       kk=1
-23166 if(.not.(kk.le.nx))goto 23168
-      i=1
-23169 if(.not.(i.le.nxis))goto 23171
-      muwk(i) = ddot (nqd, wt(1,kk), 1, qdrs(1,i,kk), 1)
-23170 i=i+1
-      goto 23169
-23171 continue
+23169 if(.not.(kk.le.nx))goto 23171
       i=1
 23172 if(.not.(i.le.nxis))goto 23174
-      j=i
-23175 if(.not.(j.le.nxis))goto 23177
-      vwk(i,j) = 0.d0
-      k=1
-23178 if(.not.(k.le.nqd))goto 23180
-      vwk(i,j) = vwk(i,j) + wt(k,kk) * qdrs(k,i,kk) * qdrs(k,j,kk)
-23179 k=k+1
-      goto 23178
-23180 continue
-      vwk(i,j) = vwk(i,j) - muwk(i) * muwk(j)
-23176 j=j+1
-      goto 23175
-23177 continue
-      muwk(i) = ddot (nqd, wt0(1,kk), 1, qdrs(1,i,kk), 1) - muwk(i)
+      muwk(i) = ddot (nqd, wt(1,kk), 1, qdrs(1,i,kk), 1)
 23173 i=i+1
       goto 23172
 23174 continue
-      call daxpy (nxis, xxwt(kk), muwk, 1, mu, 1)
-      call daxpy (nxis*nxis, xxwt(kk), vwk, 1, v, 1)
-23167 kk=kk+1
-      goto 23166
-23168 continue
-      mumax = dabs(mu(idamax(nxis, mu, 1)))
       i=1
-23181 if(.not.(i.le.nxis))goto 23183
-      jpvt(i) = 0
-23182 i=i+1
+23175 if(.not.(i.le.nxis))goto 23177
+      j=i
+23178 if(.not.(j.le.nxis))goto 23180
+      vwk(i,j) = 0.d0
+      k=1
+23181 if(.not.(k.le.nqd))goto 23183
+      vwk(i,j) = vwk(i,j) + wt(k,kk) * qdrs(k,i,kk) * qdrs(k,j,kk)
+23182 k=k+1
       goto 23181
 23183 continue
+      vwk(i,j) = vwk(i,j) - muwk(i) * muwk(j)
+23179 j=j+1
+      goto 23178
+23180 continue
+      muwk(i) = ddot (nqd, wt0(1,kk), 1, qdrs(1,i,kk), 1) - muwk(i)
+23176 i=i+1
+      goto 23175
+23177 continue
+      call daxpy (nxis, xxwt(kk), muwk, 1, mu, 1)
+      call daxpy (nxis*nxis, xxwt(kk), vwk, 1, v, 1)
+23170 kk=kk+1
+      goto 23169
+23171 continue
+      mumax = dabs(mu(idamax(nxis, mu, 1)))
+      i=1
+23184 if(.not.(i.le.nxis))goto 23186
+      jpvt(i) = 0
+23185 i=i+1
+      goto 23184
+23186 continue
       call dmcdc (v, nxis, nxis, cdnew, jpvt, infowk)
-23184 continue
+23187 continue
       call dcopy (nxis, mu, 1, cdnew, 1)
       call dprmut (cdnew, nxis, jpvt, 0)
       call dtrsl (v, nxis, nxis, cdnew, 11, infowk)
@@ -500,69 +510,69 @@ C Output from Public domain Ratfor, version 1.01
       call dprmut (cdnew, nxis, jpvt, 1)
       call daxpy (nxis, 1.d0, cd, 1, cdnew, 1)
       kk=1
-23187 if(.not.(kk.le.nx))goto 23189
+23190 if(.not.(kk.le.nx))goto 23192
       i=1
-23190 if(.not.(i.le.nqd))goto 23192
+23193 if(.not.(i.le.nqd))goto 23195
       wtnew(i,kk) = dexp (ddot (nxis, qdrs(i,1,kk), nqd, cdnew, 1) + off
      *set(i,kk))
-23191 i=i+1
+23194 i=i+1
+      goto 23193
+23195 continue
+      call dscal (nqd, 1.d0/dasum(nqd,wtnew(1,kk),1), wtnew(1,kk), 1)
+23191 kk=kk+1
       goto 23190
 23192 continue
-      call dscal (nqd, 1.d0/dasum(nqd,wtnew(1,kk),1), wtnew(1,kk), 1)
-23188 kk=kk+1
-      goto 23187
-23189 continue
       if((flag.eq.0).or.(flag.eq.2))then
       rklnew = 0.d0
       kk=1
-23195 if(.not.(kk.le.nx))goto 23197
+23198 if(.not.(kk.le.nx))goto 23200
       tmp = 0.d0
       i=1
-23198 if(.not.(i.le.nqd))goto 23200
+23201 if(.not.(i.le.nqd))goto 23203
       tmp = tmp + dlog(wt0(i,kk)/wtnew(i,kk)) * wt0(i,kk)
-23199 i=i+1
+23202 i=i+1
+      goto 23201
+23203 continue
+      rklnew = rklnew + xxwt(kk) * tmp
+23199 kk=kk+1
       goto 23198
 23200 continue
-      rklnew = rklnew + xxwt(kk) * tmp
-23196 kk=kk+1
-      goto 23195
-23197 continue
       endif
       if(flag.eq.1)then
       call dset (nxis, 0.d0, cd, 1)
       call dset (nqd*nx, 1.d0/dfloat(nqd), wt, 1)
       rkl = 0.d0
       kk=1
-23203 if(.not.(kk.le.nx))goto 23205
+23206 if(.not.(kk.le.nx))goto 23208
       tmp = 0.d0
       i=1
-23206 if(.not.(i.le.nqd))goto 23208
+23209 if(.not.(i.le.nqd))goto 23211
       tmp = tmp + dlog(wt0(i,kk)/wt(i,kk)) * wt0(i,kk)
-23207 i=i+1
+23210 i=i+1
+      goto 23209
+23211 continue
+      rkl = rkl + xxwt(kk) * tmp
+23207 kk=kk+1
       goto 23206
 23208 continue
-      rkl = rkl + xxwt(kk) * tmp
-23204 kk=kk+1
-      goto 23203
-23205 continue
       iter = 0
-      goto 23186
+      goto 23189
       endif
       if(flag.eq.3)then
-      goto 23186
+      goto 23189
       endif
       if(rklnew-rkl.lt.1.d1*(1.d0+dabs(rkl))*mchpr)then
-      goto 23186
+      goto 23189
       endif
       call dscal (nxis, .5d0, mu, 1)
       if(dabs(mu(idamax(nxis, mu, 1))/mumax).lt.1.d1*mchpr)then
-      goto 23186
+      goto 23189
       endif
-23185 goto 23184
-23186 continue
+23188 goto 23187
+23189 continue
       if(flag.eq.1)then
       flag = 2
-      goto 23164
+      goto 23167
       endif
       if(flag.eq.3)then
       info = 1
@@ -570,17 +580,17 @@ C Output from Public domain Ratfor, version 1.01
       endif
       disc = 0.d0
       kk=1
-23219 if(.not.(kk.le.nx))goto 23221
+23222 if(.not.(kk.le.nx))goto 23224
       i=1
-23222 if(.not.(i.le.nqd))goto 23224
+23225 if(.not.(i.le.nqd))goto 23227
       disc = dmax1 (disc, dabs(wt(i,kk)-wtnew(i,kk))/(1.d0+dabs(wt(i,kk)
      *)))
-23223 i=i+1
+23226 i=i+1
+      goto 23225
+23227 continue
+23223 kk=kk+1
       goto 23222
 23224 continue
-23220 kk=kk+1
-      goto 23219
-23221 continue
       disc = dmax1 (disc, (mumax/(1.d0+dabs(rkl)))**2)
       disc0 = dmax1 ((mumax/(1.d0+rkl))**2, dabs(rkl-rklnew)/(1+dabs(rkl
      *)))
@@ -588,53 +598,53 @@ C Output from Public domain Ratfor, version 1.01
       call dcopy (nqd*nx, wtnew, 1, wt, 1)
       rkl = rklnew
       if(disc0.lt.prec)then
-      goto 23165
+      goto 23168
       endif
       if(disc.lt.prec)then
-      goto 23165
+      goto 23168
       endif
       if(iter.lt.maxiter)then
-      goto 23164
+      goto 23167
       endif
       if(flag.eq.0)then
       call dset (nxis, 0.d0, cd, 1)
       call dset (nqd*nx, 1.d0/dfloat(nqd), wt, 1)
       rkl = 0.d0
       kk=1
-23233 if(.not.(kk.le.nx))goto 23235
+23236 if(.not.(kk.le.nx))goto 23238
       tmp = 0.d0
       i=1
-23236 if(.not.(i.le.nqd))goto 23238
+23239 if(.not.(i.le.nqd))goto 23241
       tmp = tmp + dlog(wt0(i,kk)/wt(i,kk)) * wt0(i,kk)
-23237 i=i+1
+23240 i=i+1
+      goto 23239
+23241 continue
+      rkl = rkl + xxwt(kk) * tmp
+23237 kk=kk+1
       goto 23236
 23238 continue
-      rkl = rkl + xxwt(kk) * tmp
-23234 kk=kk+1
-      goto 23233
-23235 continue
       iter = 0
       flag = 2
       else
       info = 2
-      goto 23165
+      goto 23168
       endif
-23164 goto 23163
-23165 continue
+23167 goto 23166
+23168 continue
       rkl = 0.d0
       kk=1
-23239 if(.not.(kk.le.nx))goto 23241
+23242 if(.not.(kk.le.nx))goto 23244
       tmp = 0.d0
       i=1
-23242 if(.not.(i.le.nqd))goto 23244
+23245 if(.not.(i.le.nqd))goto 23247
       tmp = tmp + dlog(wt0(i,kk)/wt(i,kk)) * wt0(i,kk)
-23243 i=i+1
+23246 i=i+1
+      goto 23245
+23247 continue
+      rkl = rkl + xxwt(kk) * tmp
+23243 kk=kk+1
       goto 23242
 23244 continue
-      rkl = rkl + xxwt(kk) * tmp
-23240 kk=kk+1
-      goto 23239
-23241 continue
       wt(1,1) = rkl
       return
       end
